/*
	* Copyright (c) 2021-11 by ELECTRONIC ASSEMBLY <technik@lcd-module.de>
	*
	* SmallProtcoll implementation ESP32
	*
	* This file is free software; you can redistribute it and/or modify
	* it under the terms of either the GNU General Public License version 2
	* or the GNU Lesser General Public License version 2.1, both as
	* published by the Free Software Foundation.
*/

#ifndef UNITFT_H
#define UNITFT_H

#include <SPI.h>
class HardwareSerial;


#define BAUDUNITFT 921600
#define BAUDUNITFT_DIRECT 921600

#define DC1 0x11
#define DC2 0x12

#define ACK  0x06
#define NACK 0x15

#define ESC  0x1B

#define SENDREPEAT 3
#define TIMEOUT 			2000
#define TIMEOUTBUFFERFULL	 100

#define RECBUFSIZE 512

#define INTERFACE_RS232 0
//SPI not tested until now
#define INTERFACE_SPI   1
//IIC not coded until now
#define INTERFACE_IIC   2

class unitft
{
//Public access
public:
	void initialize_rs232(int baud, byte p_sbuf); //Select RS232 as interface with baudrate and SBUF-Pin
	void initialize_rs232_newBaud(int baud); //change module's baudrate for faster connection
	void initialize_spi(SPISettings settings, byte p_sbuf); //Select SPI as interface with Mode, Clockrate and SBUF-Pin
	
	boolean sendDataPacket(const char *dat, unsigned int len); //SmallProtocol packet to uniTFT
	
	void check_sendBuffer(); //retrieves data from uniTFT
	
	int dataAvailable(); //available data from uniTFT
	byte getByte(); //get one byte from uniTFT
	void getString(char *str, int maxlen=0xFFFF); //get ascii string from uniTFT
	void getString(char16_t *str, int maxlen=0xFFFF); //get unicode string from uniTFT
	void resetReceiveBuf();	//reset buffer and dispose all previous received bytes
	
	//Direct access to RS232 serial port (used for bluetooth or wifi direct access)
	int rs232_direct_available();
	int rs232_direct_read(byte *buf, int maxlen);
	byte rs232_direct_read();
	boolean rs232_direct_write(char dat);

//Private functions
private:
	HardwareSerial *uniSerial = NULL; //pointer to RS232 serial hardware port
	SPIClass *V_SPI = NULL; //popinter to SPI serial hardware port
	
	byte p_sbuf; //portpin SBUF
	
	byte interface; //variable to remember active serial port (RS232/SPI/IIC)
	
	byte sendpacket[255]; //Buffer for protocol packet
	byte receivepacket[255]; //Buffer for protocol packet
	unsigned int sendBufferReady;
	unsigned int receiveBufferFree;
	
	char recBuf[RECBUFSIZE]; //Receive buffer for received bytes from module. Ringbuffer
	char *p_recBuf;
	int rec_nRead;
	int rec_nWrite;
	
	void initialize(byte p_sbuf); //Protcol initialize
	void checkReceiveBufferStatus(); //Protcol information
	
	boolean sendDataPacket(const char *dat, unsigned int len, int rep); //SmallProtocol packet to uniTFT, repetitions
	byte sendPacket(const char *dat, byte len, byte type, int repmax = SENDREPEAT); //Send protcol packet
	byte recPacket(); //Receive protcol packet
	
	//Wrapper functions for sending and receiving
	void out(byte *dat, byte len);
	byte rec(); //rec bytes
	boolean rec_buf(byte *buf, int len);
	
	//RS232 functions for sending and receiving
	void out_rs232(byte *dat, byte len);
	byte rec_rs232();
	boolean rec_buf_rs232(byte *buf, int len);

	//SPI functions for sending and receiving
	void out_spi(byte *dat, byte len);
	byte rec_spi();
	boolean rec_buf_spi(byte *buf, int len);
};
#endif //#define UNITFT_H

extern unitft uniTFT; //Class declaration for use in other classes.

	
		